//+------------------------------------------------------------------+
//|                                                 Risk Manager.mq5 |
//|                        Copyright 2015, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//|                                            		  Anatoly Sergeev |
//|                                            		       12/06/2014 |
//+------------------------------------------------------------------+
#include<Trade\Trade.mqh>
#property copyright "Anatoly Sergeev"
#property link      "Anatoly.Sergeev@ya.ru"
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum OrderFilling
  {
   type_1,         // /
   type_2,         // /
   type_3,         // 
  };

input double risk= 5.00;                  //    %:
input double riskTrade = 0.00;            //    %:
input double riskTrail = 0.00;            //-    %:
input OrderFilling typeFilling = type_1;  //  :

CTrade  tr;            //  CTrade
MqlDateTime dt;        //Enum     
double balance;        //   
double maxDayProfit;
string prefix;         //  
//+------------------------------------------------------------------+
//|                                              |
//+------------------------------------------------------------------+
void OnInit()
  {
   if(!TerminalInfoInteger(TERMINAL_TRADE_ALLOWED))
     {
      Alert("       !");   //   
        }else{
      if(!MQLInfoInteger(MQL_TRADE_ALLOWED))
        {
         Alert("       ",__FILE__);   //   
        }
     }

   prefix=IntegerToString(AccountInfoInteger(ACCOUNT_LOGIN))+"_";

   tr.SetDeviationInPoints(10000);            //      /

   switch(typeFilling)
     {                         //  
      case type_1:
         tr.SetTypeFilling(ORDER_FILLING_FOK);
         break;
      case type_2:
         tr.SetTypeFilling(ORDER_FILLING_IOC);
         break;
      case type_3:
         tr.SetTypeFilling(ORDER_FILLING_RETURN);
         break;
     }

   checkBalance();    //    
   dayRisk();         // 
  }
//+------------------------------------------------------------------+
//|                                                    |
//+------------------------------------------------------------------+
void OnTick()
  {
   checkBalance();
   if(dayRisk())
     {
      if(riskTrade > 0.0)tradeRisk();
      if(riskTrail > 0.0)trailingStop();
     }
  }
//+------------------------------------------------------------------+
//|                                       |
//+------------------------------------------------------------------+
void checkBalance()
  {
   TimeCurrent(dt);

   if(GlobalVariableCheck(prefix+"Balance") && GlobalVariableCheck(prefix+"Time")) //  ?
     {
      if(dt.day_of_year>GlobalVariableGet(prefix+"Time")) //  ,      
        {
         GlobalVariableSet(prefix+"Time",dt.day_of_year);
         GlobalVariableSet(prefix+"Balance",AccountInfoDouble(ACCOUNT_BALANCE));
        }
        }else{                                                                             //. .
      GlobalVariableSet(prefix+"Balance",AccountInfoDouble(ACCOUNT_BALANCE));
      GlobalVariableSet(prefix+"Time",dt.day_of_year);
     }
  }
//+------------------------------------------------------------------+
//|                                               |
//+------------------------------------------------------------------+
bool dayRisk()
  {
   balance=GlobalVariableGet(prefix+"Balance");      // 
   double currentRiskPercent=(balance -(AccountInfoDouble(ACCOUNT_BALANCE)+AccountInfoDouble(ACCOUNT_PROFIT)))/(balance/100);

   if(currentRiskPercent>0) //
     {
      if(currentRiskPercent>=risk) //  .   
        {
         closedAll();                  //  
         Comment("\n  : ",DoubleToString(risk,2),"%\n",
                 isOnTradeRisk(),
                 isOnTrail(),
                 "   : ",DoubleToString(balance,2)," ",AccountInfoString(ACCOUNT_CURRENCY),"\n",
                 " : -",DoubleToString(currentRiskPercent,2),"%\n",
                 " !    !");
         return(false);                  // 
           }else{                        //     
         Comment("\n  : ",DoubleToString(risk,2),"%\n",
                 isOnTradeRisk(),
                 isOnTrail(),
                 "   : ",DoubleToString(balance,2)," ",AccountInfoString(ACCOUNT_CURRENCY),"\n",
                 " : -",DoubleToString(currentRiskPercent,2));
         return(true);
        }
        }else{                              //
      currentRiskPercent=MathAbs(currentRiskPercent);
      Comment("\n  : ",DoubleToString(risk,2),"%\n",
              isOnTradeRisk(),
              isOnTrail(),
              "   : ",DoubleToString(balance,2)," ",AccountInfoString(ACCOUNT_CURRENCY),"\n",
              " : +",DoubleToString(currentRiskPercent,2));
      return(true);
     }
  }
//+------------------------------------------------------------------+
//|                                                |
//+------------------------------------------------------------------+
void tradeRisk()
  {
   double tradeRiskPercent;
   string symbl;
   balance=GlobalVariableGet(prefix+"Balance");            // 

   if(PositionsTotal()>0) //   ?
     {
      for(int loop=0; PositionsTotal()>=loop; loop++)
        {
         symbl= PositionGetSymbol(loop);                    //  
         if(symbl != "")                                    // 
           {
            if(PositionGetDouble(POSITION_PROFIT)<0)
              {      //  
               tradeRiskPercent=MathAbs(PositionGetDouble(POSITION_PROFIT)/(balance/100));      // %
               if(tradeRiskPercent>=riskTrade) // %       
                 {
                  if(tr.PositionClose(symbl))Print("  ",symbl," .    !");      // 
                 }
              }
           }
        }
     }
  }
//+------------------------------------------------------------------+
//|                                 |
//+------------------------------------------------------------------+
bool closedAll()
  {
   string symbl;
   ulong ticket;

   if(PositionsTotal()>0)
     {
      for(int loop=0; PositionsTotal()>=loop; loop++)
        {
         symbl=PositionGetSymbol(loop);
         if(symbl!="")
           {
            if(tr.PositionClose(symbl))Print("  ",symbl," .");
           }
        }
     }

   if(OrdersTotal()>0)
     {
      for(int loop=0; OrdersTotal()>=loop; loop++)
        {
         ticket=OrderGetTicket(loop);
         if(ticket!=0)
           {
            if(tr.OrderDelete(ticket))Print("  #",ticket," .");
           }
        }
     }

   if(PositionsTotal()==0 && OrdersTotal()==0)
     {
      return(true);
        }else{
      return(false);
     }
  }
//+------------------------------------------------------------------+
//|   tradeRisk()                             |
//+------------------------------------------------------------------+
string isOnTradeRisk()
  {
   if(riskTrade>0.0)
     {
      return("   : " + DoubleToString(riskTrade, 2) + "%\n");
     }
   return("");
  }
//+------------------------------------------------------------------+
//|   -                          |
//+------------------------------------------------------------------+
string isOnTrail()
  {
   if(riskTrail>0.0)
     {
      return("-  : " + DoubleToString(riskTrail, 2) + "%\n");
     }
   return("");
  }
//+------------------------------------------------------------------+
//|-                                       |
//+------------------------------------------------------------------+
void trailingStop()
  {
   bool flag=false;
   double currentRiskPercent=((AccountInfoDouble(ACCOUNT_BALANCE)+AccountInfoDouble(ACCOUNT_PROFIT))-balance)/(balance/100);

   if(currentRiskPercent>=riskTrail && currentRiskPercent>=maxDayProfit)
     {
      maxDayProfit=currentRiskPercent;
      flag=true;
     }
   if(currentRiskPercent<=maxDayProfit-riskTrail && flag)
     {
      if(closedAll())
        {
         Alert(" -   !");
         flag=false;
         maxDayProfit=0;
        }
     }
  }
//+------------------------------------------------------------------+
//|END                                                               |
//+------------------------------------------------------------------+
